//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This example uses glutGameMode to create a fullscreen
/// 		openGL application. This is probably what you would
/// 		want for your own game.
///
/// 		It is very important to remember that when you exit
/// 		the application you call glutLeaveGameMode() so that
/// 		the OS returns the screen resolution back to the
/// 		way it originally was.
//------------------------------------------------------------

#include <stdlib.h>
#include <GL/glut.h>

//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	if (h==0)
		h=1;

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	gluPerspective(45,(float)w/h,0.1,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------	OnDraw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// set the camera position
	gluLookAt(	1,1,10,	//	eye pos
				0,0,0,	//	aim point
				0,1,0);	//	up direction

	// draw a simple teapot
	glutWireTeapot(1);

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {
	// enable depth testing
	glEnable(GL_DEPTH_TEST);
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
	// make sure we leave game mode when we exit!!
	glutLeaveGameMode();
}

//------------------------------------------------------------	OnKeyPress()
//
void OnKeyPress(unsigned char key,int,int) {
	// escape to exit.
	if (key == 27)
		exit(EXIT_SUCCESS);
}

//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);

	// set the size of the fullscreen window. Normally you do this
	// by creating a text string. for example, "640x480:24@60" requests
	// a 640 x 480 resolution, a colour depth of 24 bit at a refresh
	// rate of 60 hz.
	//
	// Normally i prefer not to use this version since it's nicer to
	// leave the screen refresh at whatever the user has set. ie, a
	// string "bpp=32;" will leave the resolution as it is, but ensure
	// a 32bit openGL window. This longer form is un-supported on linux
	// however. Doh.
	//
	#ifndef WIN32
		glutGameModeString("1024x768:32@60");
	#else
		glutGameModeString("width=1024;height=768;bpp=32;");
	#endif

	// enter game mode
	glutEnterGameMode();

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// set the function to handle changes in screen size
	glutKeyboardFunc(OnKeyPress);
	
	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
